import os
import sys
import re
from astropy.io import fits
import reflex
import importlib

try :
    am = importlib.import_module('auto_molecule')
    iu = importlib.import_module('inst_utils')
except ImportError:
    sys.path.insert(0, '%s/instruments' %(os.path.dirname(__file__)))
    am = importlib.import_module('auto_molecule')
    iu = importlib.import_module('inst_utils')
except Exception as e:
    raise(e)

# molecfit_wkf generic methods...
'''
# ------------------------------------------------------------------------------------------
# This is a pass-through version of the check_format method
# It is commented out as an example of an "optional" method.
def check_format( files, output_dir ):
    output_files=list()
    for file in files:
        output_files.append(reflex.FitsFile(file.name, file.category, None, file.purposes ))
    return output_files
'''
# ------------------------------------------------------------------------------------------
def get_object(header=None) :
    return header.get('OBJECT','UNKNOWN')
# ------------------------------------------------------------------------------------------
def dataset_chooser_keywords() :
    return 'INSTRUME,OBJECT,INS.PATH,INS.MODE,PRO.CATG'
# ------------------------------------------------------------------------------------------
def set_inst_setup(header=None) :
    """
    Return the instrument setup.
    Generally a string built from various (primary) header keywords
    that identifies the instrument setup of the current science file.
    Different values of inst_setup will indicate when check_idp must set more than just
    file-format variables, usually indicates a change in WL-setup, e.g. VIS --> NIR for XSHOOTER
    """
    return 'UNKNOWN'
# ------------------------------------------------------------------------------------------
def check_format(files, output_dir):
    """
    Create a molecfit compatible input file from an incompatible file:
    e.g. extract a 1-d spectrum from a 2-d spectrum (for example by extracting the fibre with
    the highest S/N (GIRAFFE))

    And, because molecfit works in TOPOCENTRIC+VACUUM (TOPO_VAC == TV), create a
    binary table format version of the input science with WAVE_TV, this means that
    WAVE_INLCUDE WLs will be consistent for all data, instead of having to be adjusted
    for each data-set depedning on RV (e.g. for ESPRESSO...)

    Concerning WAVELENGHTH_FRAME:
    WAVELENGTH_FRAME = VAC|AIR|VAC_RV
        VAC = vacuum+topocentric
        AIR = air+topocentric
        VAC_RV = vacuum+barycentric
    Molecfit handles internally these three cases, however because
    molecfit works in the TOPOCENTRIC frame (i.e. the earth reference frame)
    for convenience ALL should be transformed first to TOPOCENTRIC+VACUUM
    (TOPO_VAC == TV) in check_format()

    The best way to do this is to:
        i) re-format input into a Binary Table (if necessary)
        ii) create the WL column WAVE_TV in addition to the WAVE (or whatever
            it is called) column of the original data...

    inst_utils provides the following utility functions:
        bary_to_topo(wl [<any>], barycorr [km/s])
        topo_to_bary(wl [<any>], barycorr [km/s])
        air_to_vac(wl [micron], wlg) wl either in micron, or give wlg (wl_to_micron conversion factor)
        vac_to_air(wl [micron], wlg) wl either in micron, or give wlg (wl_to_micron conversion factor)
        heli_to_topo( wl [<any>], gcorr [km/s], hcorr [km/s] )
        topo_to_heli( wl [<any>], gcorr [km/s], hcorr [km/s] )
        heli_air_to_topo_vac( wl [<any>], gcorr [km/s], hcorr [km/s], wlg )
        topo_to_heli(vac_to_air( wl [<any>], gcorr [km/s], hcorr [km/s], wlg )


    espresso.py provides an example of adding the WAVE_TV column
    """
    return files
# ------------------------------------------------------------------------------------------
wlg = 1.0e-03  # nm --> micron :: WLG_TO_MICRON
def set_parameters(header=None, hdu=None, previous_inst_setup='None', files=None) :
    """
    Set the basic, data-format-independent parameters for all inst_settings for this instrument
    """

    return {}
    """
    ## Typically the following are set here...
    wframe='AIR'
    clam='NULL'
    cflux='NULL'
    cdflux='NULL'
    fit_tel_back="False"
    fit_cont="1"
    cont_poly = int(1)
    varkern="FALSE"
    ftol = 1e-10
    xtol = 1e-10

    return {
        'WLG_TO_MICRON': wlg,
        'WAVELENGTH_FRAME': wframe,
        'FTOL': ftol,
        'XTOL': xtol,
        'COLUMN_LAMBDA': clam,
        'COLUMN_FLUX': cflux,
        'COLUMN_DFLUX': cdflux, 
        'FIT_TELESCOPE_BACKGROUND': fit_tel_back,
        'FIT_CONTINUUM': fit_cont,
        'CONTINUUM_N': cont_poly,
        'VARKERN': varkern,
    }
    """
# ------------------------------------------------------------------------------------------
def check_idp(hdu, previous_inst_setup='None', files=None, ParameterInitialization=False, recipe=None, ) :
    """
    Set any parameters needed for the specific input file format (generally either FITS-Image
    or ESO-IDP stlye binary table).
    Also set wavelength ranges and molecules if instrument setup changes.
    """
    return {}
# ------------------------------------------------------------------------------------------
def recombine_idps( files=None, orig_files=None ) :
    """
    Apply molecfit_correct results back to original molecfit-incompatible file.
    e.g. apply the correction calcuated for the highest S/N fibre 1-d spectrum back onto each
    fibre of the original input 2-d GIRAFFE spectrum 
    """
    return files
# ------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------
# -----   INST specific methods   ----------------------------------------------------------
# ------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------
def inst_specific_method() :
    pass
