# AREA A1: the import statements
# import statements needed by esoreflex
import reflex
import sys
from argparse import ArgumentParser
import json
import os
import importlib
import json
import copy
import re

# Import statements needed by the 
# algorithm to execute
from astropy.io import fits as fits
import numpy as np

# END OF AREA A1
# AREA A2: the algorithm(s).
# END OF AREA A2
# AREA A3: the interface.
# This is the main Python script. It contains 2 general parts and a
# customized part, that is responsible of calling the desired
# reduction function.
# ------------------------------------------------------------------------------------------
def do_stuff( instrument, default_oca_dir, user_oca_dir, wkf_oca_extn ):

    user_oca="%s/%s%s" %(default_oca_dir,instrument,wkf_oca_extn)
    # First try to find an OCA file for this instrument in the User's directories...
    if user_oca_dir.lower() == "_default_" :
        for path_dir in ["%s/KeplerData/workflows/MyWorkflows" %(os.environ.get('HOME')),] :
            _user_oca="%s/my_molecfit/%s%s" %(path_dir,instrument,wkf_oca_extn)
            if path_dir in [os.environ.get('PWD'),] :
                ## Trying to get python to check in the dir from which esoreflex is
                ## started, but this doesn't work because the python scripts are
                ## started in the bookkeeping directories...
                _user_oca="%s/%s%s" %(path_dir,instrument,wkf_oca_extn)
            if os.path.exists(_user_oca) :
                user_oca=_user_oca
    else :
        _user_oca="%s/%s%s" %(user_oca_dir,instrument,wkf_oca_extn)
        if os.path.exists(_user_oca) :
            user_oca=_user_oca

    try:
        # First try for a user version -- this allows users to "easily" set up support
        # for new instruments placing their python scripts somewhere a 'my_molecfit' directory
        # in a sys.path directory, e.g.:
        # on macOS:
        #   $HOME/Library/Python/3.6/lib/python/site-packages/my_molecfit
        # or on standard Linuxs
        #   $HOME/.local/lib/python3.6/site-packages/my_molecfit
        sys.path.insert(0, "%s/KeplerData/workflows/MyWorkflows/my_molecfit" %(os.environ.get('HOME')))
        inst = importlib.import_module('%s' %(instrument.lower()))
    except ImportError :
        try:
            # Then try the version that should be there in an installation...
            # When installed, the <inst>.py scripts are copied into the same directory
            # as this script...
            inst = importlib.import_module('%s' %(instrument.lower()))
        except ImportError :
            # then try the version that should be in instruments/ because in the
            # source the <inst>.py scripts are still in the instruments/ directory
            try:
                sys.path.insert(0, '%s/instruments' %(os.path.dirname(__file__)))
                inst = importlib.import_module('%s' %(instrument.lower()))
            except ImportError :
                raise Exception('%s not (yet?) supported.' % (instrument))
    except Exception as e :
        raise Exception(e)

    try:
        dataset_chooser_keywords=inst.dataset_chooser_keywords()
    except AttributeError :
        dataset_chooser_keywords='INSTRUME,OBJECT,MJD-OBS,INS.PATH,INS.MODE,PRO.CATG'
    except ImportError as e :
        raise e

    return user_oca, dataset_chooser_keywords
# ------------------------------------------------------------------------------------------
if __name__ == '__main__':

    # ***  PART P1: Input/output ports ***

    #Define inputs/outputs
    parser = reflex.ReflexIOParser()
    parser.add_option("--instrument", dest="instrument")
    parser.add_option("--default_oca_dir", dest="default_oca_dir")
    parser.add_option("--user_oca_dir", dest="user_oca_dir")
    parser.add_option("--wkf_oca_extn", dest="wkf_oca_extn")

    parser.add_output("--instrument_oca", dest="instrument_oca")
    parser.add_output("--dataset_chooser_keywords", dest="dataset_chooser_keywords")

    inputs = parser.get_inputs()
    outputs = parser.get_outputs()
    instrument = inputs.instrument
    default_oca_dir = inputs.default_oca_dir
    user_oca_dir = inputs.user_oca_dir
    wkf_oca_extn = inputs.wkf_oca_extn

    #Define the list of outputs

    #Get the name of the output directory
    pattern = '^--products-dir='
    for arg in sys.argv:
        m=re.match(pattern, arg)
        if m is not None :
            output_dir = re.sub(pattern, '', arg)

    # ***  END OF PART P1 ***

    # ***  PART P2: customized part***
    instrument_oca, dataset_chooser_keywords = do_stuff(
        instrument,
        default_oca_dir,
        user_oca_dir,
        wkf_oca_extn
    )

    # *** END OF PART P2***

    # *** PART P3: broadcast products ***
  
    # After the script is completed, the list of products is
    # broadcasted to the output sof.
    outputs.instrument_oca = instrument_oca
    outputs.dataset_chooser_keywords = dataset_chooser_keywords

    # Broadcast outputs on the extra output port.
    # broadcast outputs:
    parser.write_outputs()
    sys.exit()
    # ***  END OF PART P3 ***
    # ***  END OF AREA A3***    
